@@ -1,23 +1,25 @@
 # These functions are
-# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2018 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
 
 
 
-cqo <- function(formula,
-                family, data = list(),
-                weights = NULL, subset = NULL, na.action = na.fail,
-                etastart = NULL, mustart = NULL, coefstart = NULL,
-                control = qrrvglm.control(...),
-                offset = NULL,
-                method = "cqo.fit",
-                model = FALSE, x.arg = TRUE, y.arg = TRUE,
-                contrasts = NULL,
-                constraints = NULL,
-                extra = NULL,
-                smart = TRUE, ...) {
+cqo <-
+  function(formula,
+           family = stop("argument 'family' needs to be assigned"),
+           data = list(),
+           weights = NULL, subset = NULL, na.action = na.fail,
+           etastart = NULL, mustart = NULL, coefstart = NULL,
+           control = qrrvglm.control(...),
+           offset = NULL,
+           method = "cqo.fit",
+           model = FALSE, x.arg = TRUE, y.arg = TRUE,
+           contrasts = NULL,
+           constraints = NULL,
+           extra = NULL,
+           smart = TRUE, ...) {
   dataname <- as.character(substitute(data))  # "list" if no data =
   function.name <- "cqo"
 
@@ -59,7 +61,7 @@ cqo <- function(formula,
   w <- model.weights(mf)
   if (!length(w)) {
     w <- rep_len(1, nrow(mf))
-  } else if (ncol(as.matrix(w)) == 1 && any(w < 0))
+  } else if (NCOL(w) == 1 && any(w < 0))
     stop("negative weights not allowed")
 
   if (is.character(family))
@@ -85,10 +87,11 @@ cqo <- function(formula,
     if (control$trace && (control$Bestof>1))
     cat(paste("\n========================= Fitting model", tries,
                 "=========================\n"))
-    onefit <- cqo.fitter(x = x, y = y, w = w, offset = offset,
-              etastart = etastart, mustart = mustart, coefstart = coefstart,
-              family = family, control = control, constraints = constraints,
-              extra = extra, Terms = mt, function.name = function.name, ...)
+    onefit <-
+      cqo.fitter(x = x, y = y, w = w, offset = offset,
+          etastart = etastart, mustart = mustart, coefstart = coefstart,
+          family = family, control = control, constraints = constraints,
+          extra = extra, Terms = mt, function.name = function.name, ...)
     deviance.Bestof[tries] <- if (length(onefit$crit.list$deviance))
       onefit$crit.list$deviance else onefit$crit.list$loglikelihood
     if (tries == 1 ||
@@ -111,7 +114,7 @@ cqo <- function(formula,
     "call"         = ocall,
     "coefficients" = fit$coefficients,
     "constraints"  = fit$constraints,
-    "criterion"    = fit$crit.list,  # list("deviance" = min(deviance.Bestof)),
+    "criterion" = fit$crit.list,  # list("deviance"=min(deviance.Bestof)),
     "dispersion"   = 1,
     "family"       = fit$family,
     "misc"         = fit$misc,
