@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2018 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -44,7 +44,7 @@
 
 
            ...) {
-                           
+
 
 
 
@@ -137,11 +137,11 @@
 
 
   yn1 <- if (length(dimnames(y)[[1]])) dimnames(y)[[1]] else
-             paste(iprefix, 1:nrow(y), sep = "")
+             param.names(iprefix, nrow(y))
   warn.save <- options()$warn
   options(warn = -3)  # Suppress the warnings (hopefully, temporarily)
   if (any(!is.na(as.numeric(substring(yn1, 1, 1)))))
-    yn1 <- paste(iprefix, 1:nrow(y), sep = "")
+    yn1 <- param.names(iprefix, nrow(y))
   options(warn = warn.save)
 
 
@@ -186,7 +186,7 @@
     for (ii in cindex) {
       temp6.mat <- modmat.col[, ii, drop = FALSE]
          Hlist[[paste(cprefix, ii, sep = "")]] <- temp6.mat
-      .rcim.df[[paste(cprefix, ii, sep = "")]] <- rep(1, nrow(y))
+      .rcim.df[[paste(cprefix, ii, sep = "")]] <- rep_len(1, nrow(y))
     }
 
 
@@ -208,9 +208,9 @@
   str1 <- paste(if (has.intercept) "~ 1 + " else "~ -1 + ", rprefix,
                 as.character(min.row.val),  # "2",
                 sep = "")
-  
 
-  if (nrow(y) > 2) 
+
+  if (nrow(y) > 2)
     str1 <- paste(str1,
                   paste(rprefix, rindex[-1], sep = "", collapse = " + "),
                   sep = " + ")
@@ -271,7 +271,7 @@
     kmat1[which.linpred, 1] <- 1
     kmat0 <- (diag(M1))[, -which.linpred, drop = FALSE]
 
-    for (ii in 1:length(Hlist)) {
+    for (ii in seq_along(Hlist)) {
       Hlist[[ii]] <- kronecker(Hlist[[ii]], kmat1)
     }
     if (has.intercept)
@@ -299,7 +299,7 @@
              constraints = Hlist,
              offset = offset.matrix,
              weights = if (length(weights))
-                       weights else rep(1, length = nrow(y)),
+                       weights else rep_len(1, nrow(y)),
              ...,
              control = mycontrol, data = .rcim.df )
   } else {
@@ -309,7 +309,7 @@
              constraints = Hlist,
              offset = offset.matrix,
              weights = if (length(weights))
-                       weights else rep(1, length = nrow(y)),
+                       weights else rep_len(1, nrow(y)),
              ...,
              control = mycontrol, data = .rcim.df )
   }
@@ -320,7 +320,7 @@
   answer <- if (summary.arg) {
     if (Rank > 0) {
       summary.rrvglm(as(answer, "rrvglm"), h.step = h.step)
-    } else { 
+    } else {
       summary(answer)
     }
   } else {
@@ -385,26 +385,26 @@ setMethod("summary", "rcim",
   mat <- as.matrix(mat)
   RRR <- dim(mat)[1]
   CCC <- dim(mat)[2]
-    
+
   rnames <- if (is.null(rownames(mat))) {
-    paste("X", 1:RRR, sep = "")
+    param.names("X", RRR)
   } else {
     rownames(mat)
   }
 
   cnames <- if (is.null(colnames(mat))) {
-    paste("Y", 1:CCC, sep = "")
+    param.names("Y", CCC)
   } else {
     colnames(mat)
   }
 
-  r.index <- if (is.character(rbaseline))  
+  r.index <- if (is.character(rbaseline))
                which(rownames(mat) == rbaseline) else
                      if (is.numeric(rbaseline)) rbaseline else
-                         stop("argement 'rbaseline' must be numeric", 
+                         stop("argement 'rbaseline' must be numeric",
                                "or character of the level of row")
- 
-  c.index <- if (is.character(cbaseline))  
+
+  c.index <- if (is.character(cbaseline))
                which(colnames(mat) == cbaseline) else
                      if (is.numeric(cbaseline)) cbaseline else
                          stop("argement 'cbaseline' must be numeric",
@@ -424,11 +424,11 @@ setMethod("summary", "rcim",
 
   new.rnames <- rnames[c(r.index:RRR,
                          if (r.index > 1) 1:(r.index - 1) else NULL)]
-  new.cnames <- cnames[c(c.index:CCC, 
+  new.cnames <- cnames[c(c.index:CCC,
                          if (c.index > 1) 1:(c.index - 1) else NULL)]
   colnames(yswap) <- new.cnames
   rownames(yswap) <- new.rnames
-  
+
   yswap
 }
 
@@ -461,7 +461,7 @@ setMethod("summary", "rcim",
      no.warning = FALSE,
      ...) {
 
- 
+
   nparff <- if (is.numeric(object@family@infos()$M1)) {
     object@family@infos()$M1
   } else {
@@ -491,16 +491,16 @@ setMethod("summary", "rcim",
 
 
   orig.raxisl  <- rownames(object@y)
-  orig.caxisl  <- colnames(object@y) 
+  orig.caxisl  <- colnames(object@y)
   if (is.null(orig.raxisl))
     orig.raxisl <- as.character(1:nrow(object@y))
   if (is.null(orig.caxisl))
     orig.caxisl <- as.character(1:ncol(object@y))
-    
-  roweff.orig <- 
+
+  roweff.orig <-
   roweff <- orig.roweff[c(rfirst:last.r,
                           if (rfirst > 1) 1:(rfirst-1) else NULL)]
-  coleff.orig <- 
+  coleff.orig <-
   coleff <- orig.coleff[c(cfirst:last.c,
                           if (cfirst > 1) 1:(cfirst-1) else NULL)]
 
@@ -512,17 +512,17 @@ setMethod("summary", "rcim",
   raxisl <- orig.raxisl[c(rfirst:last.r,
                           if (rfirst > 1) 1:(rfirst-1) else NULL)]
 
-  caxisl <- orig.caxisl[c(cfirst:last.c, 
+  caxisl <- orig.caxisl[c(cfirst:last.c,
                           if (cfirst > 1) 1:(cfirst-1) else NULL)]
 
 
   if (any(which.plots == 1, na.rm = TRUE)) {
-    plot(roweff, type = rtype, 
+    plot(roweff, type = rtype,
          axes = FALSE, col = rcol, main = rmain,
          sub  = rsub, xlab = rxlab, ylab = rylab, ...)
 
-    axis(1, at = 1:length(raxisl),
-         cex.lab = rcex.lab,  
+    axis(1, at = seq_along(raxisl),
+         cex.lab = rcex.lab,
          cex.axis = rcex.axis,
          labels = raxisl)
     axis(2, cex.lab = rcex.lab, ...)  # las = rlas)
@@ -533,16 +533,16 @@ setMethod("summary", "rcim",
 
 
   if (any(which.plots == 2, na.rm = TRUE)) {
-    plot(coleff, type = ctype, 
+    plot(coleff, type = ctype,
          axes = FALSE, col = ccol, main = cmain,  # lwd = 2, xpd = FALSE,
          sub  = csub, xlab = cxlab, ylab = cylab, ...)
 
-    axis(1, at = 1:length(caxisl),
+    axis(1, at = seq_along(caxisl),
          cex.lab = ccex.lab,
          cex.axis = ccex.axis,
          labels = caxisl)
     axis(2, cex.lab = ccex.lab, ...)  # las = clas)
-    
+
     if (hline0)
       abline(h = 0, lty = hlty, col = hcol, lwd = hlwd)
   }
@@ -633,11 +633,11 @@ setMethod("plot", "rcim",
 
   rownames.mat <- rownames(mat)
   if (length(rownames.mat) != nrow(mat))
-    rownames.mat <- paste(rprefix, 1:nrow(mat), sep = "")
+    rownames.mat <- param.names(rprefix, nrow(mat))
 
   colnames.mat <- colnames(mat)
   if (length(colnames.mat) != ncol(mat))
-    colnames.mat <- paste(cprefix, 1:ncol(mat), sep = "")
+    colnames.mat <- param.names(cprefix, ncol(mat))
 
 
   newrn <- if (roffset > 0)
@@ -752,7 +752,7 @@ Confint.nb1 <- function(nb1, level = 0.95) {
 
 
 
-  myvec <- cbind(c(-1, 1, rep(0, len = nrow(myvcov) - 2)))
+  myvec <- cbind(c(-1, 1, rep_len(0, nrow(myvcov) - 2)))
   (se.mydiff <- sqrt(t(myvec) %*%  myvcov %*%  myvec))
 
   ci.mydiff <- mydiff + c(-1, 1) * qnorm(1 - (1 - level)/2) * se.mydiff
@@ -773,7 +773,8 @@ Confint.nb1 <- function(nb1, level = 0.95) {
 
 plota21 <- function(rrvglm2, show.plot = TRUE, nseq.a21 = 31,
                     se.eachway = c(5, 5),  # == c(LHS, RHS),
-                    trace.arg = TRUE, ...) {
+                    trace.arg = TRUE,
+                    lwd = 2, ...) {
 
 
 
@@ -859,7 +860,7 @@ plota21 <- function(rrvglm2, show.plot = TRUE, nseq.a21 = 31,
            col = "darkorange", lty = "dashed")
 
     abline(v = a21.hat +  c(-1, 1) * 1.96 * SE.a21.hat,
-           col = "gray50", lty = "dashed", lwd = 2.0)
+           col = "gray50", lty = "dashed", lwd = lwd)
 
   }  # End of (show.plot)
 
@@ -1059,7 +1060,7 @@ plota21 <- function(rrvglm2, show.plot = TRUE, nseq.a21 = 31,
                                  covmat[jlocal, jlocal] -
                                  covmat[ilocal, jlocal] * 2
 
-  diag(allvcov) <- rep(1.0, length = LLL)  # Any positive value should do
+  diag(allvcov) <- rep_len(1.0, LLL)  # Any positive value should do
 
 
   wmat   <- matrix(1.0, LLL, LLL)
@@ -1090,7 +1091,7 @@ WorstErrors <- function(qv.object) {
   reducedForm <- function(covmat, qvmat) {
     nlevels <- dim(covmat)[1]
     firstRow <- covmat[1, ]
-    ones <- rep(1, nlevels)
+    ones <- rep_len(1, nlevels)
     J <- outer(ones, ones)
     notzero <- 2:nlevels
     r.covmat <- covmat + (firstRow[1]*J) -
@@ -1124,7 +1125,7 @@ IndentPrint <- function(object, indent = 4, ...) {
   try(print(object, ...))
   sink()
   close(tc)
-  indent <- paste(rep(" ", indent), sep = "", collapse = "")
+  indent <- paste(rep_len(" ", indent), sep = "", collapse = "")
   cat(paste(indent, zz, sep = ""), sep = "\n")
 }
 
@@ -1154,7 +1155,7 @@ summary.qvar <- function(object, ...) {
       is.matrix(object@extra$attributes.y$estimates))
     names( estimates) <- rownames(object@extra$attributes.y$estimates)
   if (!length(names(estimates)))
-    names( estimates) <- paste("Level", 1:length(estimates), sep = "")
+    names( estimates) <- param.names("Level", length(estimates))
 
 
   regularVar <- c(object@extra$attributes.y$regularVar)
@@ -1284,8 +1285,7 @@ qvplot   <-  function(object,
   if (length(level.names) == length(estimates)) {
     names(estimates) <- level.names
   } else if (!length(names(estimates)))
-    names(estimates) <- paste("Level", 1:length(estimates),
-                              sep = "")
+    names(estimates) <- param.names("Level", length(estimates))
 
 
 
@@ -1362,8 +1362,8 @@ qvplot   <-  function(object,
 
 
   if (any(slotNames(object) == "post")) {
-    object@post$estimates  <- estimates 
-    object@post$xvalues    <- xvalues  
+    object@post$estimates  <- estimates
+    object@post$xvalues    <- xvalues
     if (is.numeric(interval.width)) {
       object@post$tails <- tails
       object@post$tops  <- tops
