@@ -1,34 +1,36 @@
 # These functions are
-# Copyright (C) 1998-2016 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2018 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
 
 
 
-vglm <- function(formula,
-                 family, data = list(), 
-                 weights = NULL, subset = NULL, na.action = na.fail,
-                 etastart = NULL, mustart = NULL, coefstart = NULL,
-                 control = vglm.control(...), 
-                 offset = NULL, 
-                 method = "vglm.fit",
-                 model = FALSE, x.arg = TRUE, y.arg = TRUE,
-                 contrasts = NULL, 
-                 constraints = NULL,
-                 extra = list(), 
-                 form2 = NULL, 
-                 qr.arg = TRUE, smart = TRUE, ...) {
+vglm <-
+  function(formula,
+           family = stop("argument 'family' needs to be assigned"),
+           data = list(),
+           weights = NULL, subset = NULL, na.action = na.fail,
+           etastart = NULL, mustart = NULL, coefstart = NULL,
+           control = vglm.control(...),
+           offset = NULL,
+           method = "vglm.fit",
+           model = FALSE, x.arg = TRUE, y.arg = TRUE,
+           contrasts = NULL,
+           constraints = NULL,
+           extra = list(),
+           form2 = NULL,
+           qr.arg = TRUE, smart = TRUE, ...) {
   dataname <- as.character(substitute(data))  # "list" if no data=
   function.name <- "vglm"
 
 
   ocall <- match.call()
 
-  if (smart) 
+  if (smart)
     setup.smart("write")
 
-  if (missing(data)) 
+  if (missing(data))
     data <- environment(formula)
 
   mf <- match.call(expand.dots = FALSE)
@@ -72,11 +74,11 @@ vglm <- function(formula,
     Xm2 <- retlist$Xm2
 
     if (length(Ym2)) {
-      if (nrow(as.matrix(Ym2)) != nrow(as.matrix(y)))
+      if (NROW(Ym2) != NROW(y))
         stop("number of rows of 'y' and 'Ym2' are unequal")
     }
     if (length(Xm2)) {
-      if (nrow(as.matrix(Xm2)) != nrow(as.matrix(x)))
+      if (NROW(Xm2) != NROW(x))
         stop("number of rows of 'x' and 'Xm2' are unequal")
     }
   } else {
@@ -85,12 +87,12 @@ vglm <- function(formula,
 
 
   offset <- model.offset(mf)
-  if (is.null(offset)) 
+  if (is.null(offset))
     offset <- 0 # yyy ???
   w <- model.weights(mf)
   if (!length(w)) {
     w <- rep_len(1, nrow(mf))
-  } else if (ncol(as.matrix(w)) == 1 && any(w < 0))
+  } else if (NCOL(w) == 1 && any(w < 0))
     stop("negative weights not allowed")
 
   if (is.character(family))
@@ -112,7 +114,7 @@ vglm <- function(formula,
   fit <- vglm.fitter(x = x, y = y, w = w, offset = offset,
            Xm2 = Xm2, Ym2 = Ym2,
            etastart = etastart, mustart = mustart, coefstart = coefstart,
-           family = family, 
+           family = family,
            control = control,
            constraints = constraints,
            extra = extra,
@@ -127,19 +129,19 @@ vglm <- function(formula,
   }
 
   answer <-
-  new(Class = "vglm", 
+  new(Class = "vglm",
     "assign"       = attr(x, "assign"),
     "call"         = ocall,
     "coefficients" = fit$coefficients,
     "constraints"  = fit$constraints,
     "criterion"    = fit$crit.list,
     "df.residual"  = fit$df.residual,
-    "df.total"     = fit$df.total, 
+    "df.total"     = fit$df.total,
     "dispersion"   = 1,
     "effects"      = fit$effects,
     "family"       = fit$family,
     "misc"         = fit$misc,
-    "model"        = if (model) mf else data.frame(), 
+    "model"        = if (model) mf else data.frame(),
     "R"            = fit$R,
     "rank"         = fit$rank,
     "residuals"    = as.matrix(fit$residuals),
@@ -217,23 +219,23 @@ attr(vglm, "smart") <- TRUE
 
 shadowvglm <-
         function(formula,
-                 family, data = list(), 
+                 family, data = list(),
                  weights = NULL, subset = NULL, na.action = na.fail,
                  etastart = NULL, mustart = NULL, coefstart = NULL,
-                 control = vglm.control(...), 
-                 offset = NULL, 
+                 control = vglm.control(...),
+                 offset = NULL,
                  method = "vglm.fit",
                  model = FALSE, x.arg = TRUE, y.arg = TRUE,
-                 contrasts = NULL, 
+                 contrasts = NULL,
                  constraints = NULL,
-                 extra = list(), 
+                 extra = list(),
                  qr.arg = FALSE, ...) {
     dataname <- as.character(substitute(data))  # "list" if no data=
     function.name <- "shadowvglm"
 
     ocall <- match.call()
 
-    if (missing(data)) 
+    if (missing(data))
         data <- environment(formula)
 
     mf <- match.call(expand.dots = FALSE)
@@ -247,7 +249,7 @@ shadowvglm <-
            stop("invalid 'method': ", method))
     mt <- attr(mf, "terms")
 
-    x <- y <- NULL 
+    x <- y <- NULL
 
     xlev <- .getXlevels(mt, mf)
     y <- model.response(mf, "any")  # model.extract(mf, "response")
